package com.github.jfcloud.sims.sample.automatic.datav.controller;

import com.github.jfcloud.sims.sample.automatic.datav.dto.*;
import com.github.jfcloud.sims.sample.automatic.datav.service.SimsDatavIndexService;
import com.github.jfcloud.sims.sample.automatic.datav.vo.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author jackzhou
 * @version 1.0
 * @project jfcloud-sims-cqjk
 * @description sims首页统计假数据接口
 * @date 2024/5/14 09:05:20
 */
@RequestMapping("/simsIndex")
@RestController
public class SimsDataIndexController {

    @Resource
    SimsDatavIndexService simsDatavIndexService;

    /**
     * 样本处理进度
     */
    @GetMapping("/sampleProcessingProgress")
    public SampleProcessingProgressInfo sampleProcessingProgress(SampleProcessingProgressVo sampleProcessingProgressVo) {
        sampleProcessingProgressVo.setEndDate(new Date());
        sampleProcessingProgressVo.setPendingSampleCount(100);
        sampleProcessingProgressVo.setProcessingSampleCount(128);
        sampleProcessingProgressVo.setDoneSampleCount(98);
        sampleProcessingProgressVo.setPageInfo(1);
        SampleProcessingProgressInfo sampleProcessingProgressInfo = simsDatavIndexService.sampleProcessingProgress(sampleProcessingProgressVo);
        return sampleProcessingProgressInfo;
    }

    /**
     *设备工作效率
     */
    @GetMapping("/equipmentProductivity")
    public EquipmentProductivityInfo equipmentProductivity(EquipmentProductivityVo equipmentProductivityVo){
        equipmentProductivityVo.setEndDate(new Date());
        Map<String,String> statusMap = new HashMap<>();
        statusMap.put("设备冰箱1","NORMAL");
        statusMap.put("设备冰箱2","RUNNING");
        statusMap.put("设备冰箱3","FAULT");
        statusMap.put("设备冰箱4","STOPPED");

        Map<String,Date> startDateMap = new HashMap<>();
        startDateMap.put("sam-1",new Date(2024 - 1900,5 - 1,14,15,30));
        startDateMap.put("sam-2",new Date(2024 - 1900,5 - 1,14,15,20));
        Map<String,Date> endDateMap = new HashMap<>();
        endDateMap.put("sam-1",new Date());
        endDateMap.put("sam-2",new Date());

        equipmentProductivityVo.setEquipmentOperationalStatusMap(statusMap);
        equipmentProductivityVo.setStartDateMap(startDateMap);
        equipmentProductivityVo.setEndDateMap(endDateMap);
        equipmentProductivityVo.setDailyProcessingCapacity(100);
        equipmentProductivityVo.setUtilNowProcessingCapacity(endDateMap.size());
        equipmentProductivityVo.setPageInfo(1);
        EquipmentProductivityInfo equipmentProductivityInfo = simsDatavIndexService.equipmentProductivity(equipmentProductivityVo);
        return equipmentProductivityInfo;
    }

    /**
     * 样本出入销库统计
     */
    @GetMapping("/statisticsOfSamplesInAndOutTheWarehouse")
    public StatisticsOfSamplesInAndOutTheWarehouseInfo statisticsOfSamplesInAndOutTheWarehouse(StatisticsOfSamplesInAndOutTheWarehouseVo statisticsOfSamplesInAndOutTheWarehouseVo){
        statisticsOfSamplesInAndOutTheWarehouseVo.setNumWeek(1);
        Map<String,Integer> storageMap = new HashMap<>();
        storageMap.put("4.16",1);
        storageMap.put("4.17",2);
        Map<String,Integer> fetchedMap = new HashMap<>();
        fetchedMap.put("4.16",3);
        fetchedMap.put("4.17",4);
        Map<String,Integer> sellOutMap = new HashMap<>();
        sellOutMap.put("4.16",5);
        sellOutMap.put("4.17",6);
        statisticsOfSamplesInAndOutTheWarehouseVo.setStorageCountOfWeekMap(storageMap);
        statisticsOfSamplesInAndOutTheWarehouseVo.setFetchedCountOfWeekMap(fetchedMap);
        statisticsOfSamplesInAndOutTheWarehouseVo.setSellOutCountOfWeekMap(sellOutMap);
        statisticsOfSamplesInAndOutTheWarehouseVo.setPageInfo(1);
        StatisticsOfSamplesInAndOutTheWarehouseInfo statisticsOfSamplesInAndOutTheWarehouseInfo = simsDatavIndexService.statisticsOfSamplesInAndOutTheWarehouse(statisticsOfSamplesInAndOutTheWarehouseVo);
        return statisticsOfSamplesInAndOutTheWarehouseInfo;
    }

    /**
     * 样本入库相关指标
     */
    @GetMapping("/sampleWarehousingRelatedIndicators")
    public SampleWarehousingRelatedIndicatorsInfo sampleWarehousingRelatedIndicators(SampleWarehousingRelatedIndicatorsVo sampleWarehousingRelatedIndicatorsVo){
        Date date = new Date();
        date.setHours(15);
        date.setMinutes(0);
        date.setSeconds(0);
        sampleWarehousingRelatedIndicatorsVo.setStorageStartTime(date);
        sampleWarehousingRelatedIndicatorsVo.setRobotOperationalStatus("125.9");
        Map<String,String> locationMap = new HashMap<>();
        locationMap.put("实验室1号楼1D-3c","状态1");
        sampleWarehousingRelatedIndicatorsVo.setStorageLocationMap(locationMap);
        sampleWarehousingRelatedIndicatorsVo.setStorageCount(116);
        sampleWarehousingRelatedIndicatorsVo.setNeedStorageCount(200);
        sampleWarehousingRelatedIndicatorsVo.setPageInfo(1);
        SampleWarehousingRelatedIndicatorsInfo sampleWarehousingRelatedIndicatorsInfo = simsDatavIndexService.sampleWarehousingRelatedIndicators(sampleWarehousingRelatedIndicatorsVo);
        return sampleWarehousingRelatedIndicatorsInfo;
    }

    /**
     * 样本出库相关指标
     */
    @GetMapping("/sampleOutboundRelatedIndicators")
    public SampleOutboundRelatedIndicatorsInfo sampleOutboundRelatedIndicators(SampleOutboundRelatedIndicatorsVo sampleOutboundRelatedIndicatorsVo){
        Map<String, Date> fetchedDateMap = new HashMap<>();
        fetchedDateMap.put("sam-1",new Date(2024 - 1900,5 - 1,14,15,30));
        fetchedDateMap.put("sam-2",new Date(2024 - 1900,5 - 1,14,10,30));
        sampleOutboundRelatedIndicatorsVo.setFetchedDateMap(fetchedDateMap);
        Map<String, Date> unFetchedDateMap = new HashMap<>();
        unFetchedDateMap.put("sam-1",new Date());
        unFetchedDateMap.put("sam-2",new Date());
        sampleOutboundRelatedIndicatorsVo.setUnFechedDateMap(unFetchedDateMap);
        sampleOutboundRelatedIndicatorsVo.setFetchedCount(116);
        sampleOutboundRelatedIndicatorsVo.setUrgentFetchedCount(36);
        sampleOutboundRelatedIndicatorsVo.setFetchedLocation("实验室1号楼1D-3c");
        sampleOutboundRelatedIndicatorsVo.setPageInfo(1);
        SampleOutboundRelatedIndicatorsInfo sampleOutboundRelatedIndicatorsInfo = simsDatavIndexService.sampleOutboundRelatedIndicators(sampleOutboundRelatedIndicatorsVo);
        return sampleOutboundRelatedIndicatorsInfo;
    }

    /**
     * 设备自动化预警
     */
    @GetMapping("/automaticEquipmentEarlyWarning")
    public AutomaticEquipmentEarlyWarningInfo automaticEquipmentEarlyWarning(AutomaticEquipmentEarlyWarningVo automaticEquipmentEarlyWarningVo){
        EarlyWaringVo earlyWaringVo1 = new EarlyWaringVo();
        earlyWaringVo1.setEarlyWarningDate(new Date());
        earlyWaringVo1.setEquipmentName("XXX");
        earlyWaringVo1.setEquipmentPriority("高");
        earlyWaringVo1.setEquipmentDescription("XXXXXXXXX");
        EarlyWaringVo earlyWaringVo2 = new EarlyWaringVo();
        earlyWaringVo2.setEarlyWarningDate(new Date());
        earlyWaringVo2.setEquipmentName("XXS");
        earlyWaringVo2.setEquipmentPriority("高");
        earlyWaringVo2.setEquipmentDescription("XXXXXXXXXSSS");
        List<EarlyWaringVo> earlyWaringVoList = new ArrayList<>();
        earlyWaringVoList.add(earlyWaringVo1);
        earlyWaringVoList.add(earlyWaringVo2);
        automaticEquipmentEarlyWarningVo.setEarlyWaringVoList(earlyWaringVoList);
        automaticEquipmentEarlyWarningVo.setPageInfo(1);
        AutomaticEquipmentEarlyWarningInfo automaticEquipmentEarlyWarningInfo = simsDatavIndexService.automaticEquipmentEarlyWarning(automaticEquipmentEarlyWarningVo);
        return automaticEquipmentEarlyWarningInfo;
    }

    /**
     * 样本库监测
     */
    @GetMapping("/sampleBankMonitoring")
    public SampleBankMonitoringInfo sampleBankMonitoring(SampleBankMonitoringVo sampleBankMonitoringVo){
        sampleBankMonitoringVo.setSampleCountTotal(168);
        Map<String,Integer> temperatureMap = new HashMap<>();
        temperatureMap.put("自动化库1",-80);
        temperatureMap.put("自动化库2",-80);
        temperatureMap.put("自动化库3",-80);
        temperatureMap.put("自动化库4",-80);
        Map<String,Double> humidityMap = new HashMap<>();
        humidityMap.put("自动化库1",0.36);
        humidityMap.put("自动化库2",0.36);
        humidityMap.put("自动化库3",0.36);
        humidityMap.put("自动化库4",0.36);
        sampleBankMonitoringVo.setHumidityMap(humidityMap);
        sampleBankMonitoringVo.setTemperatureMap(temperatureMap);
        sampleBankMonitoringVo.setPageInfo(1);
        SampleBankMonitoringInfo sampleBankMonitoringInfo = simsDatavIndexService.sampleBankMonitoring(sampleBankMonitoringVo);
        return sampleBankMonitoringInfo;
    }

    /**
     * 样本库数据总览
     */
    @GetMapping("/sampleDatabaseDataOverview")
    public SampleDatabaseDataOverviewInfo sampleDatabaseDataOverview(SampleDatabaseDataOverviewVo sampleDatabaseDataOverviewVo){
        sampleDatabaseDataOverviewVo.setEquipmentCountTotal(12);
        sampleDatabaseDataOverviewVo.setInventoryCount(102);
        sampleDatabaseDataOverviewVo.setInventorySpaceUtilization(0.5);
        sampleDatabaseDataOverviewVo.setSampleExpirationDateMonitoring(1);
        Map<String,Long> storageTotalMap = new HashMap<>();
        storageTotalMap.put("1月",200L);
        storageTotalMap.put("2月",400L);
        storageTotalMap.put("3月",600L);
        Map<String,Long> fetchTotalMap = new HashMap<>();
        fetchTotalMap.put("1月",100L);
        fetchTotalMap.put("2月",200L);
        fetchTotalMap.put("3月",300L);
        Map<String,Long> sellOutTotalMap = new HashMap<>();
        sellOutTotalMap.put("1月",100L);
        sellOutTotalMap.put("2月",300L);
        sellOutTotalMap.put("3月",500L);
        sampleDatabaseDataOverviewVo.setStorageTotalMap(storageTotalMap);
        sampleDatabaseDataOverviewVo.setFetchTotalMap(fetchTotalMap);
        sampleDatabaseDataOverviewVo.setSellOutTotalMap(sellOutTotalMap);
        sampleDatabaseDataOverviewVo.setPageInfo(1);
        SampleDatabaseDataOverviewInfo sampleDatabaseDataOverviewInfo = simsDatavIndexService.sampleDatabaseDataOverview(sampleDatabaseDataOverviewVo);
        return sampleDatabaseDataOverviewInfo;
    }

    /**
     * 设备类型
     */
    @GetMapping("/equipmentType")
    public EquipmentTypeInfo equipmentType(EquipmentTypeVo equipmentTypeVo){
        equipmentTypeVo.setPageInfo(2);
        Map<String,Integer> equipmentCountMap = new HashMap<>();
        equipmentCountMap.put("冰箱",20);
        equipmentCountMap.put("液氮罐",4);
        equipmentCountMap.put("AGV",4);
        equipmentTypeVo.setEquipmentCountMap(equipmentCountMap);
        EquipmentTypeInfo equipmentTypeInfo = simsDatavIndexService.equipmentType(equipmentTypeVo);
        return equipmentTypeInfo;
    }

    /**
     * 样本类型
     */
    @GetMapping("/sampleType")
    public SampleTypeInfo sampleType(SampleTypeVo sampleTypeVo){
        sampleTypeVo.setPageInfo(2);
        Map<String,Integer> sampleCountMap = new HashMap<>();
        sampleCountMap.put("DNA",164);
        sampleCountMap.put("植物细胞",217);
        sampleCountMap.put("原核细胞",80);
        sampleCountMap.put("真核细胞",100);
        sampleTypeVo.setSampleCountMap(sampleCountMap);
        SampleTypeInfo sampleTypeInfo = simsDatavIndexService.sampleType(sampleTypeVo);
        return sampleTypeInfo;
    }

    /**
     * 设备汇总
     */
    @GetMapping("/equipmentTotal")
    public EquipmentTotalInfo equipmentTotal(EquipmentTotalVo equipmentTotalVo){
        equipmentTotalVo.setPageInfo(2);
        Map<String,Integer> temperatureMap = new HashMap<>();
        temperatureMap.put("设备1",25);
        temperatureMap.put("设备2",25);
        temperatureMap.put("设备3",25);
        Map<String,Double> humidityMap = new HashMap<>();
        humidityMap.put("设备1",0.25);
        humidityMap.put("设备2",0.25);
        humidityMap.put("设备3",0.25);
        Map<String,String> statusMap = new HashMap<>();
        statusMap.put("设备1","正常");
        statusMap.put("设备2","运行");
        statusMap.put("设备3","断电");
        equipmentTotalVo.setStatusMap(statusMap);
        equipmentTotalVo.setHumidityMap(humidityMap);
        equipmentTotalVo.setTemperatureMap(temperatureMap);
        EquipmentTotalInfo equipmentTotalInfo = simsDatavIndexService.equipmentTotal(equipmentTotalVo);
        return equipmentTotalInfo;
    }

    /**
     * 样本小管出入库
     */
    @GetMapping("/sampleTubesInBase")
    public SampleTubesInBaseInfo sampleTubesInBase(SampleTubesInBaseVo sampleTubesInBaseVo){
        sampleTubesInBaseVo.setPageInfo(2);
        sampleTubesInBaseVo.setSampleCount(7610);
        sampleTubesInBaseVo.setSampleTubesCount(23410);
        sampleTubesInBaseVo.setSampleBoxCount(1861);
        sampleTubesInBaseVo.setFetchCount(200);
        sampleTubesInBaseVo.setStorageCount(400);
        SampleTubesInBaseInfo sampleTubesInBaseInfo = simsDatavIndexService.sampleTubesInBase(sampleTubesInBaseVo);
        return sampleTubesInBaseInfo;
    }

    /**
     * 设备空间利用率
     */
    @GetMapping("/deviceSpaceUtilization")
    public DeviceSpaceUtilizationInfo deviceSpaceUtilization(DeviceSpaceUtilizationVo deviceSpaceUtilizationVo){
        deviceSpaceUtilizationVo.setPageInfo(2);
        Map<String,Double> deviceSpaceMap = new HashMap<>();
        deviceSpaceMap.put("1",0.23);
        deviceSpaceMap.put("2",0.45);
        deviceSpaceMap.put("3",0.61);
        deviceSpaceUtilizationVo.setDeviceSpacemap(deviceSpaceMap);
        DeviceSpaceUtilizationInfo deviceSpaceUtilizationInfo = simsDatavIndexService.deviceSpaceUtilization(deviceSpaceUtilizationVo);
        return deviceSpaceUtilizationInfo;
    }
}
