package com.github.jfcloud.sims.sample.automatic.datav.service.impl;

import com.github.jfcloud.sims.sample.automatic.datav.dto.*;
import com.github.jfcloud.sims.sample.automatic.datav.service.SimsDatavIndexService;
import com.github.jfcloud.sims.sample.automatic.datav.utils.MyUtils;
import com.github.jfcloud.sims.sample.automatic.datav.vo.*;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * @author jackzhou
 * @version 1.0
 * @project jfcloud-sims-cqjk
 * @description
 * @date 2024/5/14 08:52:42
 */

@Service
public class SimsDatavIndexServiceImpl implements SimsDatavIndexService {
    @Override
    public SampleProcessingProgressInfo sampleProcessingProgress(SampleProcessingProgressVo sampleProcessingProgressVo) {
        SampleProcessingProgressInfo sampleProcessingProgressInfo = new SampleProcessingProgressInfo();
        Integer preSample = sampleProcessingProgressVo.getPendingSampleCount();
        Integer ingSample = sampleProcessingProgressVo.getProcessingSampleCount();
        Integer doneSample = sampleProcessingProgressVo.getDoneSampleCount();
        Double progress = doneSample.doubleValue() / (preSample + ingSample);
        sampleProcessingProgressInfo.setProgress(progress);
        sampleProcessingProgressInfo.setProcessingSampleCount(ingSample);
        sampleProcessingProgressInfo.setDoneSampleCount(doneSample);
        sampleProcessingProgressInfo.setPendingSampleCount(preSample);
        sampleProcessingProgressInfo.setEndDate(sampleProcessingProgressVo.getEndDate());
        sampleProcessingProgressInfo.setPageInfo(sampleProcessingProgressVo.getPageInfo());
        return sampleProcessingProgressInfo;
    }

    @Override
    public EquipmentProductivityInfo equipmentProductivity(EquipmentProductivityVo equipmentProductivityVo) {
        Integer capacityCount = equipmentProductivityVo.getUtilNowProcessingCapacity();
        Map<String, Date> startDateMap = equipmentProductivityVo.getStartDateMap();
        Map<String, Date> endDateMap = equipmentProductivityVo.getEndDateMap();
        Long timeDiff = getaTimeDiff(endDateMap, startDateMap);

        EquipmentProductivityInfo equipmentProductivityInfo = new EquipmentProductivityInfo();
        equipmentProductivityInfo.setEndDate(new Date());
        equipmentProductivityInfo.setEquipmentOperationalStatusMap(equipmentProductivityVo.getEquipmentOperationalStatusMap());
        equipmentProductivityInfo.setSampleProcessingTimeAvg(timeDiff/capacityCount);
        equipmentProductivityInfo.setDailyProcessingCapacity(equipmentProductivityVo.getDailyProcessingCapacity());
        equipmentProductivityInfo.setPageInfo(equipmentProductivityVo.getPageInfo());
        return equipmentProductivityInfo;
    }

    @Override
    public StatisticsOfSamplesInAndOutTheWarehouseInfo statisticsOfSamplesInAndOutTheWarehouse(StatisticsOfSamplesInAndOutTheWarehouseVo statisticsOfSamplesInAndOutTheWarehouseVo) {
        StatisticsOfSamplesInAndOutTheWarehouseInfo statisticsOfSamplesInAndOutTheWarehouseInfo = new StatisticsOfSamplesInAndOutTheWarehouseInfo();
        statisticsOfSamplesInAndOutTheWarehouseInfo.setNumWeek(statisticsOfSamplesInAndOutTheWarehouseVo.getNumWeek());
        statisticsOfSamplesInAndOutTheWarehouseInfo.setStorageCountOfWeekMap(statisticsOfSamplesInAndOutTheWarehouseVo.getStorageCountOfWeekMap());
        statisticsOfSamplesInAndOutTheWarehouseInfo.setFetchedCountOfWeekMap(statisticsOfSamplesInAndOutTheWarehouseVo.getFetchedCountOfWeekMap());
        statisticsOfSamplesInAndOutTheWarehouseInfo.setSellOutCountOfWeekMap(statisticsOfSamplesInAndOutTheWarehouseVo.getSellOutCountOfWeekMap());
        statisticsOfSamplesInAndOutTheWarehouseInfo.setPageInfo(statisticsOfSamplesInAndOutTheWarehouseVo.getPageInfo());
        return statisticsOfSamplesInAndOutTheWarehouseInfo;
    }

    @Override
    public SampleWarehousingRelatedIndicatorsInfo sampleWarehousingRelatedIndicators(SampleWarehousingRelatedIndicatorsVo sampleWarehousingRelatedIndicatorsVo) {
        SampleWarehousingRelatedIndicatorsInfo sampleWarehousingRelatedIndicatorsInfo = new SampleWarehousingRelatedIndicatorsInfo();
        Integer storageCount = sampleWarehousingRelatedIndicatorsVo.getStorageCount();
        sampleWarehousingRelatedIndicatorsInfo.setStorageCount(storageCount);
        sampleWarehousingRelatedIndicatorsInfo.setStorageStartTime(sampleWarehousingRelatedIndicatorsVo.getStorageStartTime());
        sampleWarehousingRelatedIndicatorsInfo.setRobotOperationalStatus(sampleWarehousingRelatedIndicatorsVo.getRobotOperationalStatus());
        sampleWarehousingRelatedIndicatorsInfo.setStorageLocationMap(sampleWarehousingRelatedIndicatorsVo.getStorageLocationMap());
        Integer needStorageCount = sampleWarehousingRelatedIndicatorsVo.getNeedStorageCount();
        sampleWarehousingRelatedIndicatorsInfo.setStorageCapacity(storageCount.doubleValue()/needStorageCount.doubleValue());
        sampleWarehousingRelatedIndicatorsInfo.setPageInfo(sampleWarehousingRelatedIndicatorsVo.getPageInfo());
        return sampleWarehousingRelatedIndicatorsInfo;
    }

    @Override
    public SampleOutboundRelatedIndicatorsInfo sampleOutboundRelatedIndicators(SampleOutboundRelatedIndicatorsVo sampleOutboundRelatedIndicatorsVo) {
        SampleOutboundRelatedIndicatorsInfo sampleOutboundRelatedIndicatorsInfo = new SampleOutboundRelatedIndicatorsInfo();
        sampleOutboundRelatedIndicatorsInfo.setFetchedCount(sampleOutboundRelatedIndicatorsVo.getFetchedCount());
        sampleOutboundRelatedIndicatorsInfo.setUrgentFetchedCount(sampleOutboundRelatedIndicatorsVo.getUrgentFetchedCount());
        sampleOutboundRelatedIndicatorsInfo.setFetchedLocation(sampleOutboundRelatedIndicatorsVo.getFetchedLocation());
        Map<String, Date> fetchedDateMap = sampleOutboundRelatedIndicatorsVo.getFetchedDateMap();
        Map<String, Date> unFechedDateMap = sampleOutboundRelatedIndicatorsVo.getUnFechedDateMap();
        Long timeDiff = getaTimeDiff(unFechedDateMap, fetchedDateMap);
        String fetchedTime = timeDiff / 60 + "小时" + timeDiff % 60 + "分";
        sampleOutboundRelatedIndicatorsInfo.setFetchedTime(fetchedTime);
        sampleOutboundRelatedIndicatorsInfo.setPageInfo(sampleOutboundRelatedIndicatorsVo.getPageInfo());
        return sampleOutboundRelatedIndicatorsInfo;
    }

    @Override
    public AutomaticEquipmentEarlyWarningInfo automaticEquipmentEarlyWarning(AutomaticEquipmentEarlyWarningVo automaticEquipmentEarlyWarningVo) {
        AutomaticEquipmentEarlyWarningInfo automaticEquipmentEarlyWarningInfo = new AutomaticEquipmentEarlyWarningInfo();
        automaticEquipmentEarlyWarningInfo.setEarlyWaringVoList(automaticEquipmentEarlyWarningVo.getEarlyWaringVoList());
        automaticEquipmentEarlyWarningInfo.setPageInfo(automaticEquipmentEarlyWarningVo.getPageInfo());
        return automaticEquipmentEarlyWarningInfo;
    }

    @Override
    public SampleBankMonitoringInfo sampleBankMonitoring(SampleBankMonitoringVo sampleBankMonitoringVo) {
        SampleBankMonitoringInfo sampleBankMonitoringInfo = new SampleBankMonitoringInfo();
        sampleBankMonitoringInfo.setEndDate(sampleBankMonitoringVo.getEndDate());
        sampleBankMonitoringInfo.setSampleCountTotal(sampleBankMonitoringVo.getSampleCountTotal());
        sampleBankMonitoringInfo.setTemperatureMap(sampleBankMonitoringVo.getTemperatureMap());
        sampleBankMonitoringInfo.setHumidityMap(sampleBankMonitoringVo.getHumidityMap());
        sampleBankMonitoringInfo.setPageInfo(sampleBankMonitoringVo.getPageInfo());
        return sampleBankMonitoringInfo;
    }

    @Override
    public SampleDatabaseDataOverviewInfo sampleDatabaseDataOverview(SampleDatabaseDataOverviewVo sampleDatabaseDataOverviewVo) {
        SampleDatabaseDataOverviewInfo sampleDatabaseDataOverviewInfo = new SampleDatabaseDataOverviewInfo();
        sampleDatabaseDataOverviewInfo.setEndDate(sampleDatabaseDataOverviewVo.getEndDate());
        sampleDatabaseDataOverviewInfo.setEquipmentCountTotal(sampleDatabaseDataOverviewVo.getEquipmentCountTotal());
        sampleDatabaseDataOverviewInfo.setInventoryCount(sampleDatabaseDataOverviewVo.getInventoryCount());
        sampleDatabaseDataOverviewInfo.setInventorySpaceUtilization(sampleDatabaseDataOverviewVo.getInventorySpaceUtilization());
        sampleDatabaseDataOverviewInfo.setSampleExpirationDateMonitoring(sampleDatabaseDataOverviewVo.getSampleExpirationDateMonitoring());
        sampleDatabaseDataOverviewInfo.setFetchTotalMap(sampleDatabaseDataOverviewVo.getFetchTotalMap());
        sampleDatabaseDataOverviewInfo.setStorageTotalMap(sampleDatabaseDataOverviewVo.getStorageTotalMap());
        sampleDatabaseDataOverviewInfo.setSellOutTotalMap(sampleDatabaseDataOverviewVo.getSellOutTotalMap());
        sampleDatabaseDataOverviewInfo.setPageInfo(sampleDatabaseDataOverviewVo.getPageInfo());
        return sampleDatabaseDataOverviewInfo;
    }

    @Override
    public EquipmentTypeInfo equipmentType(EquipmentTypeVo equipmentTypeVo) {
        EquipmentTypeInfo equipmentTypeInfo = new EquipmentTypeInfo();
        Map<String, Integer> equipmentCountMap = equipmentTypeVo.getEquipmentCountMap();
        equipmentTypeInfo.setEquipmentCountMap(equipmentCountMap);
        equipmentTypeInfo.setPageInfo(equipmentTypeVo.getPageInfo());
        equipmentTypeInfo.setAllEquipmentTotal(getAllEquipmentTotal(equipmentCountMap));
        return equipmentTypeInfo;
    }

    @Override
    public SampleTypeInfo sampleType(SampleTypeVo sampleTypeVo) {
        SampleTypeInfo sampleTypeInfo = new SampleTypeInfo();
        sampleTypeInfo.setPageInfo(sampleTypeVo.getPageInfo());
        sampleTypeInfo.setSampleCountMap(sampleTypeVo.getSampleCountMap());
        return sampleTypeInfo;
    }

    @Override
    public EquipmentTotalInfo equipmentTotal(EquipmentTotalVo equipmentTotalVo) {
        EquipmentTotalInfo equipmentTotalInfo = new EquipmentTotalInfo();
        equipmentTotalInfo.setPageInfo(equipmentTotalVo.getPageInfo());
        equipmentTotalInfo.setStatusMap(equipmentTotalVo.getStatusMap());
        equipmentTotalInfo.setTemperatureMap(equipmentTotalVo.getTemperatureMap());
        equipmentTotalInfo.setHumidityMap(equipmentTotalVo.getHumidityMap());
        return equipmentTotalInfo;
    }

    @Override
    public SampleTubesInBaseInfo sampleTubesInBase(SampleTubesInBaseVo sampleTubesInBaseVo) {
        SampleTubesInBaseInfo sampleTubesInBaseInfo = new SampleTubesInBaseInfo();
        sampleTubesInBaseInfo.setPageInfo(sampleTubesInBaseVo.getPageInfo());
        sampleTubesInBaseInfo.setSampleCount(sampleTubesInBaseVo.getSampleCount());
        sampleTubesInBaseInfo.setSampleTubesCount(sampleTubesInBaseVo.getSampleTubesCount());
        sampleTubesInBaseInfo.setSampleBoxCount(sampleTubesInBaseVo.getSampleBoxCount());
        sampleTubesInBaseInfo.setStorageCount(sampleTubesInBaseVo.getStorageCount());
        sampleTubesInBaseInfo.setFetchCount(sampleTubesInBaseVo.getFetchCount());
        return sampleTubesInBaseInfo;
    }

    @Override
    public DeviceSpaceUtilizationInfo deviceSpaceUtilization(DeviceSpaceUtilizationVo deviceSpaceUtilizationVo) {
        DeviceSpaceUtilizationInfo deviceSpaceUtilizationInfo = new DeviceSpaceUtilizationInfo();
        deviceSpaceUtilizationInfo.setPageInfo(deviceSpaceUtilizationVo.getPageInfo());
        deviceSpaceUtilizationInfo.setDeviceSpaceMap(deviceSpaceUtilizationVo.getDeviceSpacemap());
        return deviceSpaceUtilizationInfo;
    }

    private static Integer getAllEquipmentTotal(Map<String, Integer> equipmentCountMap) {
        Integer allEquipmentTotal = 0;
        for (String key : equipmentCountMap.keySet()) {
            allEquipmentTotal += equipmentCountMap.get(key);
        }
        return allEquipmentTotal;
    }

    private static Long getaTimeDiff(Map<String, Date> unFechedDateMap, Map<String, Date> fetchedDateMap) {
        Long timeDiff = 0L;
        for (String key : unFechedDateMap.keySet()) {
            Date fetchDate = fetchedDateMap.get(key);
            Date unFetchdate = unFechedDateMap.get(key);
            try {
                timeDiff += MyUtils.getTimeDiff(fetchDate, unFetchdate);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return timeDiff;
    }
}
